/*
 *  Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */

package org.wso2.carbon.auth.oauth.rest.api.internal;

import org.osgi.framework.BundleContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.wso2.carbon.auth.core.impl.UserNameMapperFactory;
import org.wso2.carbon.auth.user.mgt.UserStoreManager;

/**
 * Activator to register Swagger Definitions
 */
@Component(
        name = "org.wso2.carbon.auth.oauth.rest.api",
        immediate = true
)
public class BundleActivator {

    private static final Logger log = LoggerFactory.getLogger(BundleActivator.class);

    @Activate
    protected void start(BundleContext bundleContext) {

        ServiceReferenceHolder.getInstance().setUserNameMapper(UserNameMapperFactory.getInstance().getUserNameMapper());
        log.debug("Authentication Endpoint Activated");
    }

    /**
     * Get the UserStoreManager.
     *
     * @param userStoreManager UserStoreManager
     */
    @Reference(
            name = "org.wso2.carbon.auth.user.mgt.UserStoreManager",
            service = UserStoreManager.class,
            cardinality = ReferenceCardinality.MANDATORY,
            policy = ReferencePolicy.DYNAMIC,
            unbind = "unsetUserStoreManager"
    )
    protected void setUserStoreManager(UserStoreManager userStoreManager) {

        ServiceReferenceHolder.getInstance().setUserStoreManager(userStoreManager);
    }

    protected void unsetUserStoreManager(UserStoreManager userStoreManager) {

        ServiceReferenceHolder.getInstance().setUserStoreManager(null);
    }
}
